#!/usr/bin/perl

if ($#ARGV != 0) { die "usage: prog batch\n"; }

my $batch = $ARGV[0];
my $module = "sfsnet";

if ($batch < 1) { $batch = 1; }
if ($batch > 40) { $batch = 40; }

#
# first machine
#

my $mach0  = "jfk1-gblx.ron.lcs.mit.edu";
my $home0  = "/users/home/benjie/pastwatch.dist";
my $keyf0   = "$home0/exp2/$mach0/key0";
my $root0  = "/users/home/benjie/.pastwatch";
my $w0     = "$home0/exp2/$mach0/w0";
my $past0  = "$home0/bin/pastwatch";
my $lsd0   = "216.165.108.39:32221";
my $dir0   = "$home0/exp2/";
my $rpcw0  = 64;
my $base   = "$home0/exp2/base";   # only used for import

#
# second machine
#

my $mach1  = "ana1-gblx.ron.lcs.mit.edu";
my $home1  = "/users/home/benjie/pastwatch.dist";
my $keyf1   = "$home1/exp2/$mach1/key1";
my $root1  = "/users/home/benjie/.pastwatch";
my $w1     = "$home1/exp2/$mach1/w1";
my $past1  = "$home1/bin/pastwatch";
my $lsd1   = "65.164.24.58:32221";
my $dir1   = "$home1/exp2/";
my $rpcw1  = 64;

system ("date");

# first, create a view

system ("rm -rf ~/.pastwatch/test");
my $o = `setenv LSD_TCPSOCKET $lsd0; export LSD_TCPSOCKET=$lsd0; pastview test`;

my $view;
my $keyh0;
my $keyh1;

if ($o =~ /New view: (\S+)/) {
  $view = $1;
  print "view: $view\n";
}
if ($o =~ /New keyhash: (\S+)/) {
  $keyh0 = $1;
  print "key0: $keyh0\n";
}

$o = `setenv LSD_TCPSOCKET $lsd0; export LSD_TCPSOCKET=$lsd0; pastview -j $view test`;
if ($o =~ /New keyhash: (\S+)/) {
  $keyh1 = $1;
  print "key1: $keyh1\n";
}

system ("scp ~/.pastwatch/test/key.$keyh0 $mach0:$keyf0");
system ("scp ~/.pastwatch/test/key.$keyh1 $mach1:$keyf1");

# then, start working on two separate client machines

ssh ($mach0, "rm -rf $w0; mkdir -p $w0");
ssh ($mach1, "rm -rf $w1; mkdir -p $w1");

ssh ($mach0, "setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; $past0 -v $view -s $keyf0 status a");
ssh ($mach1, "setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; $past1 -v $view -s $keyf1 status a");

ssh ($mach0, "echo Import; cd $base; rm -rf pastwatch.core ; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 import -m checkin $module; echo Done Import");

ssh ($mach0, "sleep 15; echo Checkout Hot; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 checkout -d $w0/$module $module; echo Done Checkout Hot");

ssh ($mach1, "sleep 15; echo Checkout Cold; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; time $past1 -v $view -s $keyf1 checkout -d $w1/$module $module; echo Done Checkout Cold");
    
sub doit {
  my ($w, $diffs) = @_;

  if ($w == 0) {
    ssh ($mach0, "sleep 5; cd $dir0; echo ===== $diffs ======; grep -c Index $diffs; wc $diffs");
    ssh ($mach0, "cd $w0; patch -f < $dir0/$diffs");
    ssh ($mach0, "cd $w0; /bin/rm -rf `find $module -print | grep \".orig\" | xargs`");
    ssh ($mach0, "cd $w0; /bin/rm -rf `find $module -print | grep \".rej\" | xargs`");
    ssh ($mach0, "cd $w0; /bin/rm -rf $w0/zzz; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; $past0 -v $view -s $keyf0 update $module > $w0/zzz");
    ssh ($mach0, "cd $w0; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; $dir0/add-past.pl $view $keyf0 $past0 $w0/zzz");
    ssh ($mach0, "echo Commit 0; cd $w0; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 commit -m $diffs $module; echo Done Commit 0");
  }
  else {
    ssh ($mach1, "sleep 5; echo Update Warm 1; cd $w1; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; time $past1 -v $view -s $keyf1 update $module; echo Done Update Warm 1");
  }
}

sub ssh {
  my($h, $cmd) = @_;
  my $c = "ssh -x -a $h '$cmd'";
  my $status = system($c);
  return $status;
}

system ("sleep 5; date");

my $top = 100;
my $tt = 60;
while ($tt < $top) {
  for (my $i=0; $i<$batch && $tt<$top; $i++) {
    my $f = "patch-$tt";
    doit (0, $f);
    $tt++;
  }
  doit (1, "");
}

system ("date");

