

#include "sfsextauth.h"
#include "sfskeymisc.h"

class sfsdea : public sfsextauth 
{
public:
  sfsdea () : dur (0) {}
  sfsdea (int d) {
    if (d) { d += time (NULL); }
    dur = d;
  }
    
  ~sfsdea () {}
  bool loadkey (str keyloc);
  void authinit (svccb *sbp);
  void authinitcb (svccb *sbp, ref<sfsagent_auth_res> res);
  void set_name (str &n);
  sfs_time get_expire_time () { return dur; }
private:
  int dur;
  key kmeth;
};

void
sfsdea::set_name (str &n)
{
  n = "sfs dummy external agent";
}

void
sfsdea::authinit (svccb *sbp)
{
  ref<sfsagent_auth_res> res = New refcounted<sfsagent_auth_res> ();
  sfsextauth_init *aa = sbp->template getarg<sfsextauth_init> ();

  kmeth.authinit (&(aa->autharg), res, 
		  wrap (this, &sfsdea::authinitcb, sbp, res));
}

void
sfsdea::authinitcb (svccb *sbp, ref<sfsagent_auth_res> res)
{
  sbp->reply(res);
}


bool
sfsdea::loadkey (str keyloc)
{
  sfskey k;
  if (!keyloc || keyloc == "") {
    keyloc = defkey();
  }
  if (str err = sfskeyfetch (&k, keyloc, NULL)) {
    warn << err << "\n";
    return false;
  }
  kmeth.privkey = k.key;
  kmeth.name = keyloc;
  return true;
}


void 
usage(char *progname) 
{
  warn << "usage: " <<  progname << " [ -k <keyloc> ] [ -e <seconds> ]\n";
  exit (1);
}

int
main (int argc, char **argv)
{
  setprogname (argv[0]);
  sfsconst_init ();
  agent_setsock ();
  str keyloc ;
  int expires = 0;

  int ch;
  while ((ch = getopt (argc, argv, "k:e:")) != -1) {
    switch (ch) {
    case 'k':
      keyloc = optarg;
      break;
    case 'e':
      if (optarg) 
	      expires = atoi(optarg);
      break;
    default:
      usage (argv[0]);
    }
  }
      
  sfsdea   dea (expires);

  if (!dea.loadkey (keyloc) || !dea.connect ()) {
    exit (1);
  }

  dea.register_with_agent ();
  amain ();
}


