/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef CFSRW_H
#define CFSRW_H

#include "projinterface.h"
#include "cfsr_prot.h"
#include "cfsw_prot.h"

enum cfsrw_status {
  CFSRW_OK             = 0,
  CFSRWERR_ERROR       = 1,     // generic errors
  CFSRWERR_IO          = 2,
  CFSRWERR_OUTOFORDER  = 30,    // metadata (logs and proj blocks) related
  CFSRWERR_LOGERR      = 31,
  CFSRWERR_NOLOG       = 32,
  CFSRWERR_NOPROJECT   = 33,
  CFSRWERR_NOMORELOG   = 34,
  CFSRWERR_DIFFPROJECT = 35,
  CFSRWERR_PATHTOOLONG = 100,   // snapshot related
  CFSRWERR_STALE       = 101,
  CFSRWERR_NOENT       = 102,
  CFSRWERR_NOTDIR      = 103,
  CFSRWERR_NOTLINK     = 104,
  CFSRWERR_NOREVISION  = 105,
  CFSRWERR_VERCONFL    = 106,
  CFSRWERR_NONEWRECS   = 107,
  CFSRWERR_NOREPLICA   = 108,
  CFSRWERR_NOBRANCH    = 109,
  CFSRWERR_EXCLCONT    = 110,
  CFSRWERR_TRYAGAIN    = 997,   // abnormal failures
  CFSRWERR_INTR        = 998,
  CFSRWERR_NOTSUPP     = 999,
};

class cfsrw;
typedef callback<void, cfsrw_status>::ptr cbcfsrw;

class cfswcall;
typedef ptr<cfswcall> cfswcallp;
typedef callback<void, ptr<cfswcall>, cfsrw_status>::ptr cbcfsw;
extern void cfsw_memop (cfsw_proc p, cfsw_call_pair &c, bool free);

class cfswcall : public virtual refcount {
private:
  friend class cfsrw_impl;
  cfsw_proc _proc;
  cbcfsw _cb;
  cfsw_call_pair _argres;

public:
  cfswcall () : _proc (CFSWPROC_NULL), _cb (0) {}
  cfswcall (cfsw_proc p, cbcfsw cb) 
    : _proc (p), _cb (cb) { cfsw_memop (p, _argres, false); }
  ~cfswcall () { cfsw_memop (_proc, _argres, true); }

  cfsw_proc proc () const { return _proc; }
  void *arg () const { return _argres.arg; }
  void *res () const { return _argres.res; }
  void reply (cfsrw_status status) { _cb (mkref (this), status); }
};


class snapshot_obj;
class cfsrcall;
typedef ptr<cfsrcall> cfsrcallp;
typedef callback<void, ptr<cfsrcall>, cfsrw_status>::ptr cbcfsr;
extern void cfsr_memop (cfsr_proc p, cfsr_call_pair &c, bool free);

class cfsrcall : public virtual refcount
{
private:
  friend class replica_impl;
  friend class cfsr;

  cfsr_proc _proc;
  cbcfsr _cb;
  cfsr_call_pair _argres;
  ptr<snapshot_obj> _s;

public:
  bigint cr_addr;

  cfsrcall (cfsr_proc p, cbcfsr cb)
    : _proc (p), _cb (cb) { cfsr_memop (p, _argres, false); }
  ~cfsrcall () { cfsr_memop (_proc, _argres, true); }

  cfsr_proc proc () const { return _proc; }
  void *arg () const { return _argres.arg; }
  void *res () const { return _argres.res; }
  void reply (cfsrw_status status) { _cb (mkref(this), status); }
};

class replica;

class cfsrw
{
public:
  virtual ~cfsrw () {}
  static ref<cfsrw> alloc (bigint va, bigint loga, ref<sfspriv> sk, cbcfsrw cb);
  static ref<cfsrw> alloc (bigint va, cbcfsrw cb);
  virtual ptr<projinterface> getprojinf () const = 0;
  virtual void dispatch (cfswcallp c) = 0;
  virtual void dispatch (cfsrcallp c) = 0;
  virtual ptr<replica> getreplica () const = 0;
};

extern void
cfsrw_write_file (str path, bigint fh, bigint projid,
                  callback<void, cfsw_write_arg, cfsrw_status>::ref cb);

#endif

