/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef CFSW_PROTO_H
#define CFSW_PROTO_H

enum cfsw_proc {
  CFSWPROC_NULL      = 0,
  CFSWPROC_SETATTR   = 1,
  CFSWPROC_WRITE     = 2,
  CFSWPROC_CREATE    = 3,
  CFSWPROC_MKDIR     = 4,
  CFSWPROC_SYMLINK   = 5,
  CFSWPROC_RENAME    = 6,
  CFSWPROC_UNLINK    = 7,
  CFSWPROC_LINK      = 8,
  CFSWPROC_PATCH     = 9,
  CFSWPROC_TAG       = 10,
  CFSWPROC_RLOCK     = 20,
  CFSWPROC_WLOCK     = 21,
  CFSWPROC_COMMIT    = 22,
  CFSWPROC_ABORT     = 23,
};

enum cfsw_ftype {
  CFSW_REG = 0,
  CFSW_DIR = 1,
  CFSW_LNK = 2,
};

class cfsw_sattr 
{
public:
  int valid_size;
  uint64 size;
  int valid_mode;
  uint32 mode;

  cfsw_sattr () { valid_size = valid_mode = 0; size = 0; mode = 0; }

  bool parse (xin *in);
  void serialize (strbuf &sb) const {
    sint (sb, valid_size);
    squad (sb, size);
    sint (sb, valid_mode);
    sint (sb, mode);
  }
};

struct cfsw_call_pair {
  void *arg;
  void *res;
};


// CFSWPROC_SETATTR

struct cfsw_setattr_arg {
  bigint object;
  cfsw_sattr new_attributes;
};

// CFSWPROC_CREATE
// CFSWPROC_MKDIR

struct cfsw_create_arg {
  bigint dir;
  str name;
  bigint object;
  cfsw_sattr attributes;
};

struct cfsw_create_res {
  bigint object;
};

// CFSWPROC_SYMLINK

struct cfsw_symlink_arg {
  bigint dir;
  str name;
  str linkto;
  bigint object;
  cfsw_sattr attributes;
};

struct cfsw_symlink_res {
  bigint object;
};

// CFSWPROC_RENAME

struct cfsw_rename_arg {
  bigint object;
  cfsw_ftype type;
  bigint from_dir;
  str from_name;
  bigint to_dir;
  str to_name;
  bool target_exists;
  bigint target;
};

// CFSWPROC_UNLINK

struct cfsw_unlink_arg {
  bigint object;
  cfsw_ftype type;
  bigint dir;
  str name;
};

// CFSWPROC_PATCH

struct cfsw_patch_arg {
  bigint file;
  str patch;
};

// CFSWPROC_WRITE

struct cfsw_write_arg {
  bigint file;
  vec<bigint> direct;
  bigint indirect;
  bigint double_indirect;
  bigint triple_indirect;
  uint64 size;
};

// CFSWPROC_LINK

struct cfsw_link_arg {
  bigint object;
  bigint dir;
  str name;
};

// CFSWPROC_TAG

struct cfsw_tag_arg {
  bigint commit;
  str tag;
};

// CFSWPROC_RLOCK

struct cfsw_rlock_arg {
  bigint handle;
};

struct cfsw_rlock_res {
};

// CFSWPROC_WLOCK

struct cfsw_wlock_arg {
  bigint handle;
};

struct cfsw_wlock_res {
};

// CFSWPROC_ABORT

struct cfsw_abort_arg {
};

// CFSWPROC_COMMIT

struct cfsw_commit_arg {
  str message;
  str btag;
  unsigned flags;
  bigint supersede;
  bigint fork;
  long commit_time;

  cfsw_commit_arg () {
    message = "";
    btag = "";
    flags = 0;
    supersede = 0;
    fork = 0;
    commit_time = 0;
  }
};

struct cfsw_nores_res { };

#endif

