/*
 *  Copyright (C) 2005  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */
#ifndef MKLOG_H
#define MKLOG_H

#include "stdinc.h"
#include "parseopt.h"
#include "cfsrw.h"

extern "C" {
  time_t get_date (char *raw);
};

class mklog : public virtual refcount {
 public:
  // A refcounted mklog object creates a self reference in create_log,
  // and will remove the reference before calling the final_cb.

  mklog();
  void create_log (ptr <sfspriv> sk, bigint va, bool root, str timestamp, bool verbose,
		   callback<void, cfsrw_status>::ref final_cb);

 private:
  bool _verbose;
  int _work;
  ptr<mklog> _self;
  callback<void, cfsrw_status>::ptr _final_cb;

  bigint make_block_id (const void *data, int size);
  void store_block_cb (callback<void, bigint, int>::ref cb,
		       ns_stat status, bigint key);
  void store_cb (bool ispk, bigint id, int ok);
  bigint store (bigint va, const str &data, callback<void, bigint, int>::ref cb);
  void store_lptr (bigint va, const str &data,
		   callback<void, bigint, int>::ref cb, ptr <sfspriv> sk);
  void done (cfsrw_status status);

};

#endif 
