/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef SNAPSHOT_IMPL_H
#define SNAPSHOT_IMPL_H

#include "vec.h"
#include "ihash.h"
#include "stdinc.h"
#include "vervec.h"
#include "pastdbfe.h"
#include "cfs.h"
#include "cfsr.h"
#include "cfsw_record.h"

class iterinfo;
class snapshot;
class bob;

typedef struct snode {
  bigint inum;
  ptr<cfs_data> ent;
  ihash_entry<snode> slink;
  snode (bigint i, ptr<cfs_data> e) : inum (i), ent (e) {}
  ~snode () { ent = 0; }
} snode;

typedef callback<void, snode*>::ptr cbsnode_t;
typedef ihash<bigint, snode, &snode::inum, &snode::slink> snode_hash_t;

class pastdbfe;
class dbrec;

struct snapshot_info
{
  commit_info branch;
  bigint rootdir;
  bigint prevaddr;
  str btag;
  unsigned flags;
  bigint supersede;

  snapshot_info () {
    rootdir = prevaddr = 0;
    btag = ""; 
    flags = 0;
    supersede = 0;
  }
  void serialize (strbuf &sb) const;
  bool parse (xin &x);

  // valid flag values
  static const int IGNORE_BRANCH = 0x1;
  static const int MERGE_BRANCH = 0x2;
};

class snapshot : public snapshot_obj {
private:
  friend class bob;
  friend class scanner;
  friend class snapshot_loader;

  bigint thisaddr;
  snapshot_info thisinfo;

  void lookup_fmap (bigint, cbsnode_t);
  void lookup_fmap_load_block_cb (cbsnode_t, bigint, ptr<cfs_data>);

  snode_hash_t active_snodes;
  snode *update_index (bigint, ptr<cfs_data>);

public:
  static int init ();
  ptr<pastdbfe> fmap_db;
  str ptag;

  snapshot ();
  ~snapshot () {}

  bigint addr () const { return thisaddr; }
  bigint root () const { return thisinfo.rootdir; }
  bigint prev () const { return thisinfo.prevaddr; }
  commit_info branch () const { return thisinfo.branch; }
  str btag () const { return thisinfo.btag; }
  unsigned flags () const { return thisinfo.flags; }
  bigint supersede () const { return thisinfo.supersede; }
};

inline ptr<snapshot>
obj2impl (sptr s)
{
  ptr<snapshot> i = *((ptr<snapshot>*) &s);
  return i;
}

typedef callback<void, sptr, cfsrw_status>::ptr cbloadsn_t;
typedef callback<void, vec<sptr>, cfsrw_status>::ptr cbsnv_t;

class replica;

void load_snapshot (bigint sn_addr, cbloadsn_t cb);
void build_snapshot (replica_impl* r, bigint projid, const comvec& start,
                     cbsnv_t cb, vec<sptr> prevs);
void build_snapshot (replica_impl* r, bigint projid, const comvec& start,
                     cbsnv_t cb);

#endif

