#ifndef __PK_MIGRATE_H__
#define __PK_MIGRATE_H__

#include <chord.h>
#include <dhash_common.h>
#include <dhash.h>

#include "dhashcli.h"
#include "keyhash_idx.h"

#define PER_RPC_KEYS 256

class pk_migrate {
 protected:
  struct offer_wrap {
    ihash_entry <offer_wrap> link;
    bigint id;
    u_int count;
    offer_wrap(bigint _id, u_int _count) { id = _id; count = _count; }
  };
  ihash<bigint, offer_wrap, &offer_wrap::id, &offer_wrap::link, hashID> offer_count;


  dhashcli *cli;
  ptr<vnode> host_node;
  ptr<dhash_blockstore> storage;
  ptr<keyhash_idx> kidx;
  int interval;
  timecb_t *active_cb;
  unsigned minreps, maxreps, maxdown;

  int num_errors;
  int outstanding_rpcs;

  void remove_project(ptr<keyhash_idx::offer_state> s);

 public:
  pk_migrate(dhashcli *cli, ptr<vnode> host_node, 
	     ptr<dhash_blockstore> storage, ptr<keyhash_idx> kidx, 
	     int interval, int minreps, int maxreps, int maxdown);
  void stop();

  void migrate_next_proj();
  void migrate_to_replicas (ptr<keyhash_idx::offer_state> s, dhash_stat err, 
			    vec<chord_node> succs, route r);
  void migrate_offerpks_cb(ptr<location> dest, ptr<keyhash_idx::offer_state> s,
			   ref<dhash_offer_pks_res> res, clnt_stat err);
  void send_block_cb(chordID k, ptr<keyhash_idx::offer_state> s,
		     dhash_stat err, bool present);
};


#endif /* __PK_MIGRATE_H__ */

