/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include "dhash_common.h"
#include "stdinc.h"
#include "ns.h"
#include "memberlist.h"
#include "record.h"

memberlist::memberlist() :
  notify_rule("")
{}

memberlist::memberlist(const memberlist &original) :
  ptr_block(original)
{
  notify_rule = original.notify_rule;
}

memberlist::~memberlist() {}

bool
memberlist::parse (xin *in)
{
  str swver;
  int n;
  if (!in->rstr (swver))
    return false;
  if (!in->rint (n))
    return false;
  for (int i=0; i<n; i++) {
    bigint p, aux;
    str l;
    if (!in->rbigint (p) ||
	!in->rbigint (aux) ||
	!in->rstr (l))
      return false;
    if (!l || l == "")
      warn << "Pointer " << armor64A (p) << " has no label\n";
    add_entry (p, l);
  }
  if (!in->rstr (notify_rule))
    return false;
  sort ();
  return true;
}

str
memberlist::serialize ()
{
  sort (); // paranoia
  strbuf sb;
  sstr (sb, VERSION);
  sint (sb, _pointers.size ());
  for (unsigned i=0; i<_pointers.size (); i++) {
    sbigint (sb, _pointers [i]);
    sbigint (sb, 0);
    sstr (sb, _labels [i]);
  }
  sstr (sb, notify_rule);
  return sb;
}

void
memberlist::set_notify(str newrule) {
  notify_rule = newrule;
}

str
memberlist::get_notify() {
  return notify_rule;
}
